/**
 *  master
 *
 *  Node where USER is pressed to control
 *
 *  Kawin Khanobthamchai
 *  Sirut Kiatkeree
 *  Thanatwit Angsarawanee
 *
 */
#include <stdio.h>
#include <motelib/system.h>
#include <motelib/uart.h>
#include <motelib/radio.h>
#include <motelib/timer.h>
#include <motelib/led.h>
#include <motelib/button.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define start 's'
#define stop 't'
#define higher 'h'
#define lower 'l'

int degree = 0;

// convert angle in degress into value controlling servo
void servo_degree(uint16_t raw)
{
    OCR1A = (int)(600+raw*9);
}

// initialize servo at 0 degrees and its registers
void servo_init(void)
{
    DDRB = 0b0000110;
    ICR1 = 20000;

    TCCR1A = (1<<COM1A1) | (0<<COM1A0) | (1<<COM1B1) | (0<<COM1B0) | (1<<WGM11);
    TCCR1B = (1<<WGM13) | (1<<CS11);
    servo_degree(0);
    ledSet(2,1);
}

// interrupt with USER button
// press button to advance
void handler(ButtonStatus s)
{
    if (s == BUTTON_PRESSED) 
    {
        //control servo
        degree += 10;
        servo_degree(degree);
        if(degree >= 120)
        {
            degree = 0;
            ledSet(0,1);
            servo_degree(degree);
            uartWriteByte(stop);
        }
        else
        {
            uartWriteByte(start);
            ledSetValue(0);
        }
    }
    else if (s == BUTTON_RELEASED) ;
}

// receive data from UART
void receive()
{
    char buff;
    while (uartInputLen() > 0)
    {
        buff = uartReadByte();
        // if maximum, servo stops and light(2) turns on
        if (buff == higher) 
        {
            uartWriteByte(stop);
            ledSet(2,1);
        }
        // if less than maximum, servo continues rotating
        else if (buff == lower)
        {
            uartWriteByte(stop);
            //control servo
            degree += 10;
            servo_degree(degree);
            if(degree >= 120)
            {
                degree = 0;
                ledSet(0,1);
                servo_degree(degree);
            }
            else
            {
                uartWriteByte(start);
            }
        }
    }
}

void boot()
{
    servo_init();
    uartEnable(true, true);
    setLoopRoutine(receive);
    buttonSetHandler(handler);
}
