/**
 *  slave
 *
 *  Node in servo motor to receive signal
 *
 *  Kawin Khanobthamchai
 *  Sirut Kiatkeree
 *  Thanatwit Angsarawanee
 *
 */
#include <stdio.h>
#include <motelib/system.h>
#include <motelib/uart.h>
#include <motelib/radio.h>
#include <motelib/timer.h>
#include <motelib/led.h>
#include <motelib/button.h>

#define higher 'h'
#define lower 'l'
#define start 's'
#define stop 't'
#define TYPE 99

uint8_t count = 0;
int sum = 0;
int rssi_max = 0;
int status_temp;

// receive RSSI through radio
void sense(Address src, MessageType type, void *msg, uint8_t len)
{
    RadioRxStatus status;
    radioGetRxStatus(&status);
    if(type == TYPE)
    {   
        ledSet(1,1);
        count++;
        sum = sum + status.rssi;
        // average RSSI
        if(count >= 5)
        {
            status_temp = sum / count;
            count = 0;
            sum = 0;
            // if higher RSSI, send HIGH signal to master
            if (rssi_max <= status_temp)
            {
                rssi_max = status_temp;
                uartWriteByte(higher);
            }
            // send LOW signal to master otherwise
            else if (rssi_max > status_temp) 
            {
                uartWriteByte(lower);
            }
        }
    }
}

// receive data via UART from master
void receive()
{
    char buff;
    while (uartInputLen() > 0)
    {
        buff = uartReadByte();
        if (buff == start)
        {
            radioSetRxHandler(sense);
        }
        else if (buff == stop)
        {
            radioSetRxHandler(NULL);
        }
    }
}

void boot()
{
    uartEnable(true, true);
    radioSetRxHandler(NULL);
    setLoopRoutine(receive);
}
